from __future__ import absolute_import
import django

if django.VERSION[:2] >= (1, 8):
    # Django 1.9 specifically requires that models not be imported before setup,
    # which sentry.auth does, so we need to use AppConfig here.
    # Also works on 1.8.
    default_app_config = "sentry_auth_authsch.apps.Config"
else:
    # Provide backwards compatibility.
    from sentry import auth, options

    from .provider import AuthSCHProvider

    auth.register('authsch', AuthSCHProvider)




from __future__ import absolute_import, print_function

from django.conf import settings

AUTHSCH_CLIENT_ID = getattr(settings, 'AUTHSCH_CLIENT_ID', None)
AUTHSCH_CLIENT_SECRET = getattr(settings, 'AUTHSCH_CLIENT_SECRET', None)

SCOPE = "basic displayName mail admembership linkedAccounts eduPersonEntitlement"
AUTHORIZE_URL = getattr(settings, 'AUTHSCH_AUTHORIZE_URL', "https://auth.sch.bme.hu/site/login")
ACCESS_TOKEN_URL = getattr(settings, 'ACCESS_TOKEN_URL', "https://auth.sch.bme.hu/oauth2/token")
PROFILE_ENDPOINT = getattr(settings, 'PROFILE_ENDPOINT', "https://auth.sch.bme.hu/api/profile/")

ERR_INVALID_RESPONSE = "Unable to fetch user information from AuthSCH.  Please check the log."

DATA_VERSION = "1"



from __future__ import absolute_import, print_function

import re
import logging
import requests

from sentry.auth.view import AuthView, ConfigureView
from string import maketrans
from .constants import ERR_INVALID_RESPONSE, PROFILE_ENDPOINT

logger = logging.getLogger("sentry.auth.authsch")


class FetchUser(AuthView):
    def __init__(self, version, *args, **kwargs):
        self.version = version
        super(FetchUser, self).__init__(*args, **kwargs)

    def get_user_profile(self, token):
        return requests.get(
            PROFILE_ENDPOINT,
            params={'access_token': token},
            timeout=5
        ).json()

    def dispatch(self, request, helper):
        data = helper.fetch_state("data")

        try:
            # call /api/profile/?access_token=<access_token>
            # repsponse:
            # {
            #     "internal_id":"{9DA3D9BE-04CD-FD26-CB87-BAF34B0367F8}",
            #     "displayName":"Teszt Bela", "sn":"Teszt", "givenName":"Bela",
            #     "mail":"bela@tesztel.ek",
            #     "linkedAccounts": {
            #         "bme":"2214@bme.hu",
            #         "schacc":"belavagyok",
            #         "vir":"belaateszter"
            #     },
            #     "eduPersonEntitlement":[
            #         {"id":16,"name":"Simonyi K\u00e1roly Szakkoll\u00e9gium","status":"tag"},
            #         {"id":47,"name":"Koll\u00e9giumi Sz\u00e1m\u00edt\u00e1stechnikai K\u00f6r","status":"tag"},
            #         {"id":363,"name":"Buksi","status":"tag"}
            #     ]
            # }

            token = data['access_token']
            raw = self.get_user_profile(token=token)

        except Exception as exc:
            logger.exception(exc)
            return helper.error(ERR_INVALID_RESPONSE)

        try:
            userdata = self.populate_userdata(raw)
        except Exception as exc:
            logger.exception(exc)
            return helper.error(ERR_INVALID_RESPONSE)

        helper.bind_state('user', userdata)

        return helper.next_step()

    def populate_userdata(self, raw):
        userdata = dict()

        # cleaning up internal id from `{`, `}`, and `-` chars, because its unnecessary
        userdata['iid'] = re.sub('[-{}]', '', raw['internal_id'])
        userdata['schacc'] = raw['linkedAccounts']['schacc']
        userdata['full_name'] = raw['displayName']
        userdata['schmail'] = "{}@sch.bme.hu".format(userdata['schacc'])
        userdata['secondary_mail'] = raw.get('mail', None)
        userdata['raw_profile'] = raw

        return userdata.copy()


class AuthSCHConfigureView(ConfigureView):
    def dispatch(self, request, organization, auth_provider):
        return self.render('sentry_auth_authsch/configure.html')



from __future__ import absolute_import

from django.apps import AppConfig


class Config(AppConfig):
    name = "sentry_auth_authsch"

    def ready(self):
        from sentry import auth, options

        from .provider import AuthSCHProvider

        auth.register("authsch", AuthSCHProvider)




from __future__ import absolute_import, print_function

from sentry.auth.providers.oauth2 import OAuth2Callback, OAuth2Provider, OAuth2Login

from .constants import ACCESS_TOKEN_URL, AUTHSCH_CLIENT_ID, AUTHSCH_CLIENT_SECRET, AUTHORIZE_URL, SCOPE
from .views import FetchUser, AuthSCHConfigureView


DATA_VERSION = "1"


class AuthSCHProvider(OAuth2Provider):
    name = "AuthSCH"
    authorize_url = AUTHORIZE_URL
    access_token_url = ACCESS_TOKEN_URL
    client_id = AUTHSCH_CLIENT_ID
    client_secret = AUTHSCH_CLIENT_SECRET
    scope = SCOPE

    def __init__(self, version=None, **config):
        self.version = version
        super(AuthSCHProvider, self).__init__(**config)

    def get_client_id(self):
        return self.client_id

    def get_client_secret(self):
        return self.client_secret

    def get_auth_pipeline(self):
        return [
            OAuth2Login(
                client_id=self.get_client_id(),
                authorize_url=self.authorize_url,
                scope=self.scope
            ),
            OAuth2Callback(
                access_token_url=self.access_token_url,
                client_id=self.get_client_id(),
                client_secret=self.get_client_secret(),
            ),
            FetchUser(
                version=self.version
            )
        ]

    def get_refresh_token_url(self):
        return self.access_token_endpoint

    def build_identity(self, state):
        data = state["data"]
        user_data = state["user"]
        schacc = user_data["schacc"]

        return {
            "id": user_data["iid"],
            "email": user_data['schmail'],
            "email_verified": True,
            "name": user_data["full_name"],
            "nickname": schacc,
            "data": self.get_oauth_data(data),
        }

    def get_configure_view(self):
        return AuthSCHConfigureView.as_view()

    def build_config(self, state):
        return {"version": DATA_VERSION}



#!/usr/bin/env python
"""
sentry-auth-authsch
============

A Sentry integration which enables `AuthSCH <https://auth.sch.bme.hu/>`_  authentication.

:copyright: (c) 2019 by Tamas Kiss
:license: Simplified BSD, see LICENSE for more details.
"""
from setuptools import setup, find_packages


install_requires = [
    'requests>=2.20,<3.0.0'
]

tests_require = [
    'exam',
    'responses',
    'flake8>=2.0,<2.1',
    'pytest<3.6.0,>=3.5.0',
    'betamax==0.8.1',
    'sentry>=9.1.0,<10.0.0',
]

setup(
    name='sentry-auth-authsch',
    version='1.0.0',
    author='Tamas Kiss',
    author_email='tamas.kiss@kszk.bme.hu',
    url='https://github.com/thomasklein94/sentry-auth-authsch.git',
    description='AuthSCH SSO integration for Sentry',
    long_description=open('README.rst').read(),
    license='Simplified BSD',
    package_dir={'': 'src'},
    packages=find_packages('src'),
    zip_safe=False,
    install_requires=install_requires,
    extras_require={
        'tests': tests_require,
    },
    include_package_data=True,
    entry_points={
        'sentry.apps': [
            'sentry_auth_authsch = sentry_auth_authsch',
        ],
    },
    classifiers=[
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Operating System :: OS Independent',
        'Topic :: Software Development'
    ],
)
